/* -*-c-*-
 *
 * Check a path for safety
 *
 * (c) 1999 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of chkpath.
 *
 * chkpath is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * chkpath is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with chkpath; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CHECKPATH_H
#define CHECKPATH_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <limits.h>
#include <sys/types.h>

/*----- Data structures ---------------------------------------------------*/

/* --- Search request --- *
 *
 * This contains parameters from the caller to control what problems are
 * looked for, and what to do when they're found.
 */

struct checkpath {
  uid_t cp_uid;				/* Uid that's considered OK */
  gid_t cp_gid[NGROUPS_MAX + 1];	/* Array of groups that are OK */
  int cp_gids;				/* Number of groups in the array */
  int cp_verbose;			/* Verbosity level to spit up */
  unsigned cp_what;			/* What things to check for */
  void (*cp_report)(unsigned /*what*/, int /*verb*/,
		    const char */*dir*/, const char */*msg*/,
		    void */*p*/);
  void *cp_arg;				/* Argument for cp_report */
};

/* --- Flags for `@what@' fields in the above --- */

/* Problem types */
#define CP_PROBLEMS 0x1fu		/* Mask of problem bits */
#define CP_ERROR 0x01u			/* Error report */
#define CP_WRWORLD 0x02u		/* Check write by world */
#define CP_WRGRP 0x04u			/* Check write by any group */
#define CP_WROTHGRP 0x08u		/* Check write by other group */
#define CP_WROTHUSR 0x10u		/* Check write by other user */

/* Other flags */
#define CP_SYMLINK 0x100u		/* Report symbolic links */
#define CP_REPORT 0x200u		/* Make user-readable reports */
#define CP_STICKYOK 0x400u		/* Don't care if sticky is set */

/*----- Functions provided ------------------------------------------------*/

/* --- @checkpath@ --- *
 *
 * Arguments:	@const char *p@ = directory name which needs checking
 *		@const struct checkpath *cp@ = parameters for the check
 *
 * Returns:	Zero if all is well, otherwise bitmask of problems.
 *
 * Use:		Scrutinises a directory path to see what evil things other
 *		users could do to it.
 */

extern unsigned checkpath(const char */*p*/, const struct checkpath */*cp*/);

/* --- @checkpath_addgid@ --- *
 *
 * Arguments:	@struct checkpath *cp@ = pointer to block to fill in
 *		@gid_t g@ = group id to add
 *
 * Returns:	Zero if successful, nonzero if the array is full.
 *
 * Use:		Adds the group @g@ to the structure.
 */

extern int checkpath_addgid(struct checkpath */*cp*/, gid_t /*g*/);

/* --- @checkpath_setuid@ --- *
 *
 * Arguments:	@struct checkpath *cp@ = pointer to block to fill in
 *
 * Returns:	---
 *
 * Use:		Fills in the @cp_uid@ slot of the structure with the real uid
 *		of the current process.
 */

extern void checkpath_setuid(struct checkpath */*cp*/);

/* --- @checkpath_setgid@ --- *
 *
 * Arguments:	@struct checkpath *cp@ = pointer to block to fill in
 *
 * Returns:	Zero if successful, nonzero if the array is full.
 *
 * Use:		Adds the real gid of the current process to the @cp_gid@
 *		array.
 */

extern int checkpath_setgid(/*cp*/);

/* --- @checkpath_setgroups@ --- *
 *
 * Arguments:	@struct checkpath *cp@ = pointer to block to fill in
 *
 * Returns:	Zero if successful, nonzero if the array is full.
 *
 * Use:		Adds the current process's supplementary groups to the
 *		@cp_gid@ table.
 */

extern int checkpath_setgroups(struct checkpath */*cp*/);

/* --- @checkpath_setids@ --- *
 *
 * Arguments:	@struct checkpath *cp@ = pointer to block to fill in
 *
 * Returns:	---
 *
 * Use:		Fills in the user ids and things in the structure.  This is
 *		equivalent to setting @cp_gids = 0@ and then calling
 *		@_setuid@, @_setgid@ and @_setgroups@.  It can't fail.
 */

extern void checkpath_setids(struct checkpath */*cp*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
